#if !defined(INTTCW_H)
#define INTTCW_H

/***********************************************************************
   For at least the random-number generator code, we need integer types
   of guaranteed minimum size.  The C99 types uint_least32_t and
   uint_least64_t defined in <stdint.h> provide the required types, and
   it would seem simple to include that file, when available, or else
   define our own typedefs for the new types.

   Unfortunately, Standard C makes it an error to repeat a typedef
   (though it is hard to deduce this from the text of either the 1989 or
   the 1999 ISO Standards), even when it is identical to a previous
   instance.  Even more unfortunately, some vendors define those
   additional types in pre-C99 environments, but provide no way to
   record the fact that they were typedef'ed.

   The only portable solution to this mess is to define new names, which
   we choose to be uppercased versions of the new C99 types.
***********************************************************************/

#if (__STDC_VERSION__ + 0) >= 199901

#include <stdint.h>

#undef HAVE_LONG_LONG_INT
#define HAVE_LONG_LONG_INT

#if defined(__hpux)	/* HP-UX on IA-32 violates C99 by not defining uint_least64_t, sigh... */

#define uint_least64_t unsigned long long int

#endif /* defined(__hpux) */

#if defined(__mips__) && defined(__sgi__)

#undef uint_least32_t
#undef uint_least64_t
typedef unsigned long int	uint_least32_t;
typedef unsigned long long int	uint_least64_t;

#endif /* defined(__mips__) && defined(__sgi__) */

#define HAVE_UINT_LEAST64_T
#define MAX_UINT_LEAST64 ULLONG_MAX
typedef uint_least32_t UINT_LEAST32_T;
typedef uint_least64_t UINT_LEAST64_T;

#else  /* pre-C99 environment */

#include <limits.h>

#if UINT_MAX >= 0xffffffffU

#define HAVE_UINT_LEAST32_T
#define MAX_UINT_LEAST32 UINT_MAX
typedef unsigned int UINT_LEAST32_T;

#elif ULONG_MAX >= 0xffffffffUL

#define HAVE_UINT_LEAST32_T
#define MAX_UINT_LEAST32 ULONG_MAX
typedef unsigned long UINT_LEAST32_T;

#else
#error "mathcw library: host integer data types are inadequate for this software"
#endif /* UINT_MAX >= 0xffffffffU */

#if defined(ULLONG_MAX) && (ULLONG_MAX > 0xffffffffUL) && !defined(HAVE_LONG_LONG_INT)

#define HAVE_LONG_LONG_INT

#endif /* defined(ULLONG_MAX) && (ULLONG_MAX > 0xffffffffUL) && !defined(HAVE_LONG_LONG_INT) */

/* Because of preprocessor limitations on some platforms, this test is
   more complex than expected. */

#if ((ULONG_MAX + 0UL) > 0xffffffffUL) && ((ULONG_MAX + 0UL) >= 0xffffffffffffffffUL)

/*
** GNU/Linux on Alpha, AMD64, and IA-64, and OSF/1 on Alpha, have 64-bit
** long int but, curiously, Mac OS X on EM64T (== AMD64) uses a 32-bit
** long int, like many older CISC and RISC systems.  On all current
** systems, when long int is a 64-bit type, long long int (when
** supported) is identical to it: no vendor yet provides 128-bit
** integers.
*/

#define HAVE_UINT_LEAST64_T
#define MAX_UINT_LEAST64 ULONG_MAX
typedef unsigned long UINT_LEAST64_T;

#endif /* (ULONG_MAX > 0xffffffffUL) && (ULONG_MAX >= 0xffffffffffffffffUL) */

#if defined(HAVE_LONG_LONG_INT)

#if defined(ULONG_LONG_MAX) && !defined(ULLONG_MAX)

#define ULLONG_MAX	ULONG_LONG_MAX	/* map variant name to ULLONG_MAX */

#endif

#if !defined(ULLONG_MAX)

#define ULLONG_MAX 0xffffffffffffffffULL /* assume 64-bit size */

#endif

#if defined(HAVE_UINT_LEAST64_T)

/* Because of preprocessor limitations on some platforms, this test is
   more complex than expected. */

#elif ((ULONG_MAX + 0ULL) > 0xffffffffULL) && ((ULONG_MAX + 0ULL) >= 0xffffffffffffffffULL)

/* unsigned long appears to be at least a 64-bit type */
#undef HAVE_UINT_LEAST64_T
#define HAVE_UINT_LEAST64_T
#define MAX_UINT_LEAST64 ULONG_MAX
typedef unsigned long UINT_LEAST64_T;

#elif (ULLONG_MAX + 0ULL) >= 0xffffffffffffffffULL

/* unsigned long long appears to be at least a 64-bit type */
#define HAVE_UINT_LEAST64_T
#define MAX_UINT_LEAST64 ULLONG_MAX
typedef unsigned long long UINT_LEAST64_T;

#else
#error "mathcw library: host integer data types are inadequate for this software"
#endif /* defined(HAVE_UINT_LEAST64_T) */

#endif /* defined(HAVE_LONG_LONG_INT) */

#if !defined(INTMAX_MAX)
#define INTMAX_MAX	LLONG_MAX
#endif

#if !defined(INTMAX_MIN)
#define INTMAX_MIN	LLONG_MIN
#endif

#if !defined(PTRDIFF_MAX)
#define PTRDIFF_MAX	LLONG_MAX
#endif

#if !defined(PTRDIFF_MIN)
#define PTRDIFF_MIN	LLONG_MIN
#endif

#if !defined(SIZE_MAX)
#define SIZE_MAX	ULLONG_MAX
#endif

#endif /* (__STDC_VERSION__ + 0) >= 199901 */

#endif /* !defined(INTTCW_H) */
